@@ -17,7 +17,7 @@ from .jsonschema import ActionJsonSchema
 from .loaders_dumpers import check_valid_dump_format, dump_using_format, get_loader_exceptions, loaders, load_value, load_value_context, yaml_load
 from .namespace import is_meta_key, Namespace, split_key, split_key_leaf, strip_meta
 from .signatures import is_pure_dataclass, SignatureArguments
-from .typehints import ActionTypeHint
+from .typehints import ActionTypeHint, is_class_object
 from .actions import (
     ActionParser,
     ActionConfigFile,
@@ -25,6 +25,7 @@ from .actions import (
     _ActionPrintConfig,
     _ActionConfigLoad,
     _ActionLink,
+    _ActionHelpClassPath,
     _is_branch_key,
     _find_action,
     _find_action_and_subcommand,
@@ -88,13 +89,15 @@ class _ActionsContainer(SignatureArguments, argparse._ActionsContainer, LoggerPr
             return ActionParser._move_parser_actions(parser, args, kwargs)
         if 'type' in kwargs:
             if is_pure_dataclass(kwargs['type']):
-                theclass = kwargs.pop('type')
                 nested_key = re.sub('^--', '', args[0])
-                super().add_dataclass_arguments(theclass, nested_key, **kwargs)
+                super().add_dataclass_arguments(kwargs['type'], nested_key, **kwargs)
                 return _find_action(parser, nested_key)
             if ActionTypeHint.is_supported_typehint(kwargs['type']):
                 if 'action' in kwargs:
                     raise ValueError('Type hint as type does not allow providing an action.')
+                if ActionTypeHint.is_subclass_typehint(kwargs['type']):
+                    dest = re.sub('^--', '', args[0])
+                    super().add_argument(f'--{dest}.help', action=_ActionHelpClassPath(baseclass=kwargs['type']))  # type: ignore
                 kwargs['action'] = ActionTypeHint(typehint=kwargs.pop('type'), enable_path=enable_path)
         action = super().add_argument(*args, **kwargs)
         if isinstance(action, ActionConfigFile) and getattr(self, '_print_config', None) is not None:
@@ -148,6 +151,7 @@ class ArgumentParser(_ActionsContainer, argparse.ArgumentParser):
 
     formatter_class: Type[DefaultHelpFormatter]  # type: ignore
     groups: Optional[Dict[str, '_ArgumentGroup']] = None
+    _subcommands_action: Optional[_ActionSubCommands] = None
 
 
     def __init__(
@@ -210,12 +214,14 @@ class ArgumentParser(_ActionsContainer, argparse.ArgumentParser):
         caller = inspect.getmodule(inspect.stack()[1][0]).__package__
         if caller not in {'jsonargparse', 'argcomplete'}:
             raise NotImplementedError('parse_known_args not implemented to dissuade its use, since typos in configs would go unnoticed.')
+
         if args is None:
             args = sys.argv[1:]
         else:
             args = list(args)
             if not all(isinstance(a, str) for a in args):
                 self.error(f'All arguments are expected to be strings: {args}')
+        self.args = args
 
         if namespace is None:
             namespace = Namespace()
@@ -337,7 +343,7 @@ class ArgumentParser(_ActionsContainer, argparse.ArgumentParser):
         argcomplete_autocomplete(self)
 
         try:
-            cfg, unk = self.parse_known_args(args=args)
+            cfg, unk = self.parse_known_args(args=args, namespace=namespace)
             if unk:
                 self.error(f'Unrecognized arguments: {" ".join(unk)}')
 
@@ -347,7 +353,6 @@ class ArgumentParser(_ActionsContainer, argparse.ArgumentParser):
                 defaults=defaults,
                 with_meta=with_meta,
                 skip_check=_skip_check,
-                cfg_base=namespace,
                 log_message='Parsed command line arguments.',
             )
 
@@ -414,7 +419,7 @@ class ArgumentParser(_ActionsContainer, argparse.ArgumentParser):
                 env_val = env[env_var]
                 if env_val in action.choices:
                     cfg[action.dest] = subcommand = self._check_value_key(action, env_val, action.dest, cfg)
-                    pcfg = action._name_parser_map[env_val].parse_env(env=env, defaults=defaults, _skip_check=True)  # type: ignore
+                    pcfg = action._name_parser_map[env_val].parse_env(env=env, defaults=defaults, _skip_check=True)
                     for k, v in vars(pcfg).items():
                         cfg[subcommand+'.'+k] = v
         for action in actions:
@@ -658,6 +663,7 @@ class ArgumentParser(_ActionsContainer, argparse.ArgumentParser):
         cfg: Namespace,
         format: str = 'parser_mode',
         skip_none: bool = True,
+        skip_default: bool = False,
         skip_check: bool = False,
         yaml_comments: bool = False,
     ) -> str:
@@ -667,6 +673,7 @@ class ArgumentParser(_ActionsContainer, argparse.ArgumentParser):
             cfg: The configuration object to dump.
             format: The output format: ``'yaml'``, ``'json'``, ``'json_indented'``, ``'parser_mode'`` or ones added via :func:`.set_dumper`.
             skip_none: Whether to exclude entries whose value is None.
+            skip_default: Whether to exclude entries whose value is the same as the default.
             skip_check: Whether to skip parser checking.
             yaml_comments: Whether to add help content as comments. ``yaml_comments=True`` implies ``format='yaml'``.
 
@@ -686,28 +693,57 @@ class ArgumentParser(_ActionsContainer, argparse.ArgumentParser):
             with load_value_context(self.parser_mode):
                 self.check_config(cfg)
 
-        def cleanup_actions(cfg, actions, prefix=''):
-            for action in filter_default_actions(actions):
-                action_dest = prefix + action.dest
-                if (action.help == argparse.SUPPRESS and not isinstance(action, _ActionConfigLoad)) or \
-                   isinstance(action, ActionConfigFile) or \
-                   (skip_none and action_dest in cfg and cfg[action_dest] is None):
-                    cfg.pop(action_dest, None)
-                elif isinstance(action, _ActionSubCommands):
-                    cfg.pop(action_dest, None)
-                    for key, subparser in action.choices.items():
-                        cleanup_actions(cfg, subparser._actions, prefix=prefix+key+'.')
-                elif isinstance(action, ActionTypeHint):
-                    value = cfg.get(action_dest)
-                    if value is not None:
-                        value = action.serialize(value, dump_kwargs={'skip_check': skip_check, 'skip_none': skip_none})
-                        cfg.update(value, action_dest)
-
         with load_value_context(self.parser_mode):
-            cleanup_actions(cfg, self._actions)
+            dump_kwargs = {'skip_check': skip_check, 'skip_none': skip_none}
+            self._dump_cleanup_actions(cfg, self._actions, dump_kwargs)
+
+        cfg = cfg.as_dict()
+
+        if skip_default:
+            self._dump_delete_default_entries(cfg, self.get_defaults().as_dict())
 
         with formatter_context(self):
-            return dump_using_format(self, cfg.as_dict(), 'yaml_comments' if yaml_comments else format)
+            return dump_using_format(self, cfg, 'yaml_comments' if yaml_comments else format)
+
+
+    def _dump_cleanup_actions(self, cfg, actions, dump_kwargs, prefix=''):
+        skip_none = dump_kwargs['skip_none']
+        for action in filter_default_actions(actions):
+            action_dest = prefix + action.dest
+            if (action.help == argparse.SUPPRESS and not isinstance(action, _ActionConfigLoad)) or \
+               isinstance(action, ActionConfigFile) or \
+               (skip_none and action_dest in cfg and cfg[action_dest] is None):
+                cfg.pop(action_dest, None)
+            elif isinstance(action, _ActionSubCommands):
+                cfg.pop(action_dest, None)
+                for key, subparser in action.choices.items():
+                    self._dump_cleanup_actions(cfg, subparser._actions, dump_kwargs, prefix=prefix+key+'.')
+            elif isinstance(action, ActionTypeHint):
+                value = cfg.get(action_dest)
+                if value is not None:
+                    value = action.serialize(value, dump_kwargs=dump_kwargs)
+                    cfg.update(value, action_dest)
+
+
+    def _dump_delete_default_entries(self, subcfg, subdefaults):
+        for key in list(subcfg.keys()):
+            if key in subdefaults:
+                val = subcfg[key]
+                default = subdefaults[key]
+                class_object_val = None
+                if is_class_object(val):
+                    if val['class_path'] != default.get('class_path'):
+                        parser = ActionTypeHint.get_class_parser(val['class_path'])
+                        default = {'init_args': parser.get_defaults().as_dict()}
+                    class_object_val = val
+                    val = val['init_args']
+                    default = default.get('init_args')
+                if val == default:
+                    del subcfg[key]
+                elif isinstance(val, dict) and isinstance(default, dict):
+                    self._dump_delete_default_entries(val, default)
+                    if class_object_val and class_object_val.get('init_args') == {}:
+                        del class_object_val['init_args']
 
 
     def save(
@@ -891,7 +927,7 @@ class ArgumentParser(_ActionsContainer, argparse.ArgumentParser):
         cfg = Namespace()
         for action in filter_default_actions(self._actions):
             if action.default != argparse.SUPPRESS and action.dest != argparse.SUPPRESS:
-                cfg[action.dest] = action.default
+                cfg[action.dest] = deepcopy(action.default)
 
         self._logger.info('Loaded default values from parser.')
 
@@ -1205,7 +1241,7 @@ class ArgumentParser(_ActionsContainer, argparse.ArgumentParser):
             if leaf_key == action.dest:
                 return value
             subparser = action._name_parser_map[leaf_key]
-            subparser.check_config(value)  # type: ignore
+            subparser.check_config(value)
         elif isinstance(action, _ActionConfigLoad):
             if isinstance(value, str):
                 fpath = None
@@ -1370,9 +1406,9 @@ class ArgumentParser(_ActionsContainer, argparse.ArgumentParser):
         if parser_mode == 'jsonnet':
             import_jsonnet('parser_mode=jsonnet')
         self._parser_mode = parser_mode
-        if self._subparsers:
+        if self._subcommands_action:
             for subparser in self._subcommands_action._name_parser_map.values():
-                subparser.parser_mode = parser_mode  # type: ignore
+                subparser.parser_mode = parser_mode
 
 
     @property
