@@ -3,13 +3,16 @@
 import json
 import os
 import pathlib
+import re
 import sys
 import unittest
 import uuid
 import yaml
 from calendar import Calendar
+from contextlib import redirect_stderr, redirect_stdout
 from datetime import datetime
 from enum import Enum
+from gzip import GzipFile
 from io import StringIO
 from typing import Any, Callable, Dict, Iterable, List, Mapping, Optional, Sequence, Tuple, Type, Union
 from jsonargparse import ActionConfigFile, ArgumentParser, CLI, lazy_instance, Namespace, ParserError, Path
@@ -230,7 +233,7 @@ class TypeHintsTests(unittest.TestCase):
         self.assertEqual('[[[', parser.parse_args(['--any=[[['])['any'])
 
 
-    @unittest.skipIf(sys.version_info[:2] < (3, 8), 'Literal introduced in python 3.8')
+    @unittest.skipIf(not Literal, 'Literal introduced in python 3.8 or backported in typing_extensions')
     def test_Literal(self):
         parser = ArgumentParser(error_handler=None)
         parser.add_argument('--str', type=Literal['a', 'b'])
@@ -417,6 +420,8 @@ class TypeHintsTests(unittest.TestCase):
         with mock_module(MyCal) as module:
             cfg = parser.parse_args([f'--op.class_path={module}.MyCal', '--op.init_args.p1=3'], defaults=False)
             self.assertEqual(cfg.op, Namespace(class_path=f'{module}.MyCal', init_args=Namespace(p1=3)))
+            cfg = parser.parse_args(['--op.class_path', f'{module}.MyCal', '--op.init_args.p1', '3'], defaults=False)
+            self.assertEqual(cfg.op, Namespace(class_path=f'{module}.MyCal', init_args=Namespace(p1=3)))
 
 
     def test_class_type_required_params(self):
@@ -431,6 +436,85 @@ class TypeHintsTests(unittest.TestCase):
             cfg = parser.get_defaults()
             self.assertEqual(cfg.op.class_path, f'{module}.MyCal')
             self.assertEqual(cfg.op.init_args, Namespace(p1=None, p2=None))
+            self.assertRaises(ParserError, lambda: parser.parse_args([f'--op={module}.MyCal']))
+
+
+    def test_class_type_subclass_given_by_name_issue_84(self):
+        class LocalCalendar(Calendar):
+            pass
+
+        parser = ArgumentParser()
+        parser.add_argument('--op', type=Union[Calendar, GzipFile, None])
+        cfg = parser.parse_args(['--op=TextCalendar'])
+        self.assertEqual(cfg.op.class_path, 'calendar.TextCalendar')
+
+        out = StringIO()
+        parser.print_help(out)
+        for class_path in ['calendar.Calendar', 'calendar.TextCalendar', 'gzip.GzipFile']:
+            self.assertIn(class_path, out.getvalue())
+        self.assertNotIn('LocalCalendar', out.getvalue())
+
+        class HTMLCalendar(Calendar):
+            pass
+
+        with mock_module(HTMLCalendar) as module:
+            err = StringIO()
+            with redirect_stderr(err), self.assertRaises(SystemExit):
+                parser.parse_args(['--op.help=HTMLCalendar'])
+            self.assertIn('Give the full class path to avoid ambiguity', err.getvalue())
+            self.assertIn(f'{module}.HTMLCalendar', err.getvalue())
+
+
+    def test_class_type_subclass_short_init_args(self):
+        parser = ArgumentParser()
+        parser.add_argument('--op', type=Calendar)
+        cfg = parser.parse_args(['--op=TextCalendar', '--op.firstweekday=2'])
+        self.assertEqual(cfg.op.class_path, 'calendar.TextCalendar')
+        self.assertEqual(cfg.op.init_args, Namespace(firstweekday=2))
+
+
+    def test_class_type_subclass_nested_init_args(self):
+        class Class:
+            def __init__(self, cal: Calendar, p1: int = 0):
+                self.cal = cal
+
+        for full in ['init_args.', '']:
+            with self.subTest('full' if full else 'short'), mock_module(Class) as module:
+                parser = ArgumentParser()
+                parser.add_argument('--op', type=Class)
+                cfg = parser.parse_args([
+                    f'--op={module}.Class',
+                    f'--op.{full}p1=1',
+                    f'--op.{full}cal=calendar.TextCalendar',
+                    f'--op.{full}cal.{full}firstweekday=2',
+                ])
+                self.assertEqual(cfg.op.class_path, f'{module}.Class')
+                self.assertEqual(cfg.op.init_args.p1, 1)
+                self.assertEqual(cfg.op.init_args.cal.class_path, 'calendar.TextCalendar')
+                self.assertEqual(cfg.op.init_args.cal.init_args, Namespace(firstweekday=2))
+
+
+    def test_class_type_subclass_nested_help(self):
+        class Class:
+            def __init__(self, cal: Calendar, p1: int = 0):
+                self.cal = cal
+
+        parser = ArgumentParser()
+        parser.add_argument('--op', type=Class)
+
+        for pattern in [r'[\s=]', r'\s']:
+            with self.subTest('" "' if '=' in pattern else '"="'), mock_module(Class) as module:
+                out = StringIO()
+                args = re.split(pattern, f'--op.help={module}.Class --op.init_args.cal.help=TextCalendar')
+                with redirect_stdout(out), self.assertRaises(SystemExit):
+                    parser.parse_args(args)
+                self.assertIn('--op.init_args.cal.init_args.firstweekday', out.getvalue())
+
+        with self.subTest('invalid'), mock_module(Class) as module:
+            err = StringIO()
+            with redirect_stderr(err), self.assertRaises(SystemExit):
+                parser.parse_args([f'--op.help={module}.Class', '--op.init_args.p1=1'])
+            self.assertIn('Expected a nested --*.help option', err.getvalue())
 
 
     def test_invalid_init_args_in_yaml(self):
@@ -503,6 +587,31 @@ class TypeHintsTests(unittest.TestCase):
         self.assertRaises(ValueError, lambda: lazy_instance(MyClass, param='bad'))
 
 
+    def test_dump_skip_default(self):
+        class MyCalendar(Calendar):
+            def __init__(self, *args, param: str = '0', **kwargs):
+                super().__init__(*args, **kwargs)
+
+        with mock_module(MyCalendar) as module:
+            parser = ArgumentParser()
+            parser.add_argument('--g1.op1', default=123)
+            parser.add_argument('--g1.op2', default='abc')
+            parser.add_argument('--g2.op1', default=4.5)
+            parser.add_argument('--g2.op2', type=Calendar, default=lazy_instance(Calendar, firstweekday=2))
+
+            cfg = parser.get_defaults()
+            dump = parser.dump(cfg, skip_default=True)
+            self.assertEqual(dump, '{}\n')
+
+            cfg.g2.op2.class_path = f'{module}.MyCalendar'
+            dump = parser.dump(cfg, skip_default=True)
+            self.assertEqual(dump, 'g2:\n  op2:\n    class_path: jsonargparse_tests.MyCalendar\n    init_args:\n      firstweekday: 2\n')
+
+            cfg.g2.op2.init_args.firstweekday = 0
+            dump = parser.dump(cfg, skip_default=True)
+            self.assertEqual(dump, 'g2:\n  op2:\n    class_path: jsonargparse_tests.MyCalendar\n')
+
+
 class TypeHintsTmpdirTests(TempDirTestCase):
 
     def test_path(self):
